@@ -10,7 +10,8 @@ Arguments:
   <path>                Path to initialize (default is the current path)
 
 Options:
-  --force -f            Allow init inside a project subfolder
+  --force -f            Allow initialization inside a directory that is not
+                        empty, or a subdirectory of an existing project
   --help -h             Display this message
 
 brownie init is used to create new brownie projects. It creates the default
@@ -22,10 +23,10 @@ interfaces/             Interface source code
 reports/                Report files for contract analysis
 scripts/                Scripts for deployment and interaction
 tests/                  Scripts for project testing
-brownie-config.yaml     Project configuration file"""
+"""
 
 
 def main():
     args = docopt(__doc__)
-    path = project.new(args["<path>"] or ".", args["--force"])
-    notify("SUCCESS", f"Brownie environment has been initiated at {path}")
+    path = project.new(args["<path>"] or ".", args["--force"], args["--force"])
+    notify("SUCCESS", f"A new Brownie project has been initialized at {path}")
