@@ -169,6 +169,7 @@ class ContractContainer(_ContractBase):
             address: Address string of the contract.
             owner: Default Account instance to send contract transactions from.
             tx: Transaction ID of the contract creation."""
+        address = _resolve_address(address)
         contract = _find_contract(address)
         if isinstance(contract, ProjectContract):
             if contract._name == self._name and contract._project == self._project:
@@ -238,8 +239,8 @@ class ContractConstructor:
         args, tx = _get_tx(None, args)
         if not tx["from"]:
             raise AttributeError(
-                "No deployer address given. You must supply a tx dict"
-                " as the last argument with a 'from' field."
+                "Final argument must be a dict of transaction parameters that "
+                "includes a `from` field specifying the address to deploy from"
             )
         return tx["from"].deploy(
             self._parent,
@@ -292,8 +293,8 @@ class ContractConstructor:
         args, tx = _get_tx(None, args)
         if not tx["from"]:
             raise AttributeError(
-                "Contract has no owner, you must supply a tx dict"
-                " as the last argument with a 'from' field."
+                "Final argument must be a dict of transaction parameters that "
+                "includes a `from` field specifying the sender of the transaction"
             )
 
         return tx["from"].estimate_gas(
@@ -351,6 +352,7 @@ class _DeployedContractBase(_ContractBase):
         tx: TransactionReceipt of the of the tx that deployed the contract."""
 
     _reverted = False
+    _initialized = False
 
     def __init__(
         self, address: str, owner: Optional[AccountsType] = None, tx: TransactionReceiptType = None
@@ -383,6 +385,8 @@ class _DeployedContractBase(_ContractBase):
                 self._check_and_set(abi["name"], overloaded)
             getattr(self, abi["name"])._add_fn(abi, natspec)
 
+        self._initialized = True
+
     def _check_and_set(self, name: str, obj: Any) -> None:
         if name == "balance":
             warnings.warn(
@@ -425,6 +429,14 @@ class _DeployedContractBase(_ContractBase):
         except AttributeError:
             raise AttributeError(f"Contract '{self._name}' object has no attribute '{name}'")
 
+    def __setattr__(self, name: str, value: Any) -> None:
+        if self._initialized and hasattr(self, name):
+            if isinstance(getattr(self, name), _ContractMethod):
+                raise AttributeError(
+                    f"{self._name}.{name} is a contract function, it cannot be assigned to"
+                )
+        super().__setattr__(name, value)
+
     def get_method_object(self, calldata: str) -> Optional["_ContractMethod"]:
         """
         Given a calldata hex string, returns a `ContractMethod` object.
@@ -1064,8 +1076,8 @@ class _ContractMethod:
         args, tx = _get_tx(self._owner, args)
         if not tx["from"]:
             raise AttributeError(
-                "Contract has no owner, you must supply a tx dict"
-                " as the last argument with a 'from' field."
+                "Final argument must be a dict of transaction parameters that "
+                "includes a `from` field specifying the sender of the transaction"
             )
 
         return tx["from"].transfer(
@@ -1077,6 +1089,7 @@ class _ContractMethod:
             nonce=tx["nonce"],
             required_confs=tx["required_confs"],
             data=self.encode_input(*args),
+            allow_revert=tx["allow_revert"],
         )
 
     def decode_input(self, hexstr: str) -> List:
@@ -1153,8 +1166,8 @@ class _ContractMethod:
         args, tx = _get_tx(self._owner, args)
         if not tx["from"]:
             raise AttributeError(
-                "Contract has no owner, you must supply a tx dict"
-                " as the last argument with a 'from' field."
+                "Final argument must be a dict of transaction parameters that "
+                "includes a `from` field specifying the sender of the transaction"
             )
 
         return tx["from"].estimate_gas(
@@ -1270,6 +1283,7 @@ def _get_tx(owner: Optional[AccountsType], args: Tuple) -> Tuple:
         "gasPrice": None,
         "nonce": None,
         "required_confs": 1,
+        "allow_revert": None,
     }
     if args and isinstance(args[-1], dict):
         tx.update(args[-1])
@@ -1281,7 +1295,7 @@ def _get_tx(owner: Optional[AccountsType], args: Tuple) -> Tuple:
     # enable the magic of ganache's `evm_unlockUnknownAccount`
     if isinstance(tx["from"], str):
         tx["from"] = accounts.at(tx["from"], force=True)
-    elif isinstance(tx["from"], Contract):
+    elif isinstance(tx["from"], _DeployedContractBase):
         tx["from"] = accounts.at(tx["from"].address, force=True)
 
     return args, tx
