@@ -45,6 +45,13 @@ class TxHistory(metaclass=_Singleton):
             return str(self._list)
         return super().__repr__()
 
+    def __getattribute__(self, name: str) -> Any:
+        # filter dropped transactions prior to attribute access
+        items = super().__getattribute__("_list")
+        items = [i for i in items if i.status != -2]
+        setattr(self, "_list", items)
+        return super().__getattribute__(name)
+
     def __bool__(self) -> bool:
         return bool(self._list)
 
@@ -76,7 +83,9 @@ class TxHistory(metaclass=_Singleton):
         """Returns a shallow copy of the object as a list"""
         return self._list.copy()
 
-    def filter(self, key: Optional[Callable] = None, **kwargs: Dict) -> List:
+    def filter(
+        self, key: Optional[Callable] = None, **kwargs: Optional[Any]
+    ) -> List[TransactionReceipt]:
         """
         Return a filtered list of transactions.
 
@@ -190,6 +199,9 @@ class Chain(metaclass=_Singleton):
             self._block_gas_time = block["timestamp"]
         return block
 
+    def __iter__(self) -> Iterator:
+        return iter(web3.eth.getBlock(i) for i in range(web3.eth.blockNumber + 1))
+
     def new_blocks(self, height_buffer: int = 0, poll_interval: int = 5) -> Iterator:
         """
         Generator for iterating over new blocks.
@@ -315,7 +327,7 @@ class Chain(metaclass=_Singleton):
             self._redo_buffer.clear()
             self._current_id = self._snap()
 
-    def mine(self, blocks: int = 1) -> int:
+    def mine(self, blocks: int = 1, timestamp: int = None, timedelta: int = None) -> int:
         """
         Increase the block height within the test RPC.
 
@@ -323,6 +335,13 @@ class Chain(metaclass=_Singleton):
         ---------
         blocks : int
             Number of new blocks to be mined
+        timestamp : int
+            Timestamp of the final block being mined. If multiple blocks
+            are mined, they will be placed at equal intervals starting
+            at `chain.time()` and ending at `timestamp`.
+        timedelta : int
+            Timedelta for the final block to be mined. If given, the final
+            block will have a timestamp of `chain.time() + timedelta`
 
         Returns
         -------
@@ -330,9 +349,28 @@ class Chain(metaclass=_Singleton):
             Current block height
         """
         if not isinstance(blocks, int):
-            raise TypeError("blocks must be an integer value")
+            raise TypeError("`blocks` must be an integer value")
+
+        if timedelta is not None and timestamp is not None:
+            raise ValueError("Cannot use both `timestamp` and `timedelta`")
+
+        if timedelta is not None:
+            timestamp = self.time() + timedelta
+
+        if timestamp is None:
+            params: List = [[] for i in range(blocks)]
+        elif blocks == 1:
+            params = [[timestamp]]
+        else:
+            now = self.time()
+            duration = (timestamp - now) / (blocks - 1)
+            params = [[round(now + duration * i)] for i in range(blocks)]
+
         for i in range(blocks):
-            self._request("evm_mine", [])
+            self._request("evm_mine", params[i])
+
+        if timestamp is not None:
+            self.sleep(0)
 
         self._redo_buffer.clear()
         self._current_id = self._snap()
