@@ -5,6 +5,7 @@ import sys
 import threading
 import time
 from collections import OrderedDict
+from enum import IntEnum
 from hashlib import sha1
 from pathlib import Path
 from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple, Union
@@ -13,7 +14,7 @@ import black
 import requests
 from eth_abi import decode_abi
 from hexbytes import HexBytes
-from web3.exceptions import TransactionNotFound
+from web3.exceptions import TimeExhausted, TransactionNotFound
 
 from brownie._config import CONFIG
 from brownie.convert import EthAddress, Wei
@@ -26,7 +27,7 @@ from brownie.utils import color
 from brownie.utils.output import build_tree
 
 from . import state
-from .event import _decode_logs, _decode_trace
+from .event import EventDict, _decode_logs, _decode_trace
 from .web3 import web3
 
 
@@ -35,11 +36,21 @@ def trace_property(fn: Callable) -> Any:
 
     @property  # type: ignore
     def wrapper(self: "TransactionReceipt") -> Any:
-        if self.status == -1:
+        if self.status < 0:
             return None
         if self._trace_exc is not None:
             raise self._trace_exc
-        return fn(self)
+        try:
+            return fn(self)
+        except RPCRequestError as exc:
+            if web3.supports_traces:
+                # if the node client supports traces, raise the actual error
+                raise exc
+            raise RPCRequestError(
+                f"Accessing `TransactionReceipt.{fn.__name__}` on a {self.status.name.lower()} "
+                "transaction requires the `debug_traceTransaction` RPC endpoint, but the node "
+                "client does not support it or has not made it available."
+            ) from None
 
     return wrapper
 
@@ -58,6 +69,13 @@ def trace_inspection(fn: Callable) -> Any:
     return wrapper
 
 
+class Status(IntEnum):
+    Dropped = -2
+    Pending = -1
+    Reverted = 0
+    Confirmed = 1
+
+
 class TransactionReceipt:
 
     """Attributes and methods relating to a broadcasted transaction.
@@ -105,7 +123,7 @@ class TransactionReceipt:
     logs = None
     nonce = None
     sender = None
-    txid = None
+    txid: str
     txindex = None
 
     def __init__(
@@ -114,6 +132,7 @@ class TransactionReceipt:
         sender: Any = None,
         silent: bool = True,
         required_confs: int = 1,
+        is_blocking: bool = True,
         name: str = "",
         revert_data: Optional[Tuple] = None,
     ) -> None:
@@ -123,6 +142,8 @@ class TransactionReceipt:
             txid: hexstring transaction ID
             sender: sender as a hex string or Account object
             required_confs: the number of required confirmations before processing the receipt
+            is_blocking: if True, creating the object is a blocking action until the required
+                         confirmations are received
             silent: toggles console verbosity (default True)
             name: contract function being called
             revert_data: (revert string, program counter, revert type)
@@ -132,27 +153,31 @@ class TransactionReceipt:
         if isinstance(txid, bytes):
             txid = HexBytes(txid).hex()
         if not self._silent:
-            print(f"Transaction sent: {color('bright blue')}{txid}{color}")
+            print(f"\rTransaction sent: {color('bright blue')}{txid}{color}")
+
+        # this event is set once the transaction is confirmed or dropped
+        # it is used to waiting during blocking transaction actions
+        self._confirmed = threading.Event()
 
         # internal attributes
-        self._trace_exc = None
-        self._trace_origin = None
-        self._raw_trace = None
-        self._trace = None
         self._call_cost = 0
-        self._events = None
-        self._return_value = None
-        self._revert_msg = None
-        self._modified_state = None
-        self._new_contracts = None
-        self._internal_transfers = None
+        self._trace_exc: Optional[Exception] = None
+        self._trace_origin: Optional[str] = None
+        self._raw_trace: Optional[List] = None
+        self._trace: Optional[List] = None
+        self._events: Optional[EventDict] = None
+        self._return_value: Any = None
+        self._revert_msg: Optional[str] = None
+        self._dev_revert_msg: Optional[str] = None
+        self._modified_state: Optional[bool] = None
+        self._new_contracts: Optional[List] = None
+        self._internal_transfers: Optional[List[Dict]] = None
         self._subcalls: Optional[List[Dict]] = None
-        self._confirmed = threading.Event()
 
         # attributes that can be set immediately
         self.sender = sender
-        self.status = -1
-        self.txid = txid
+        self.status = Status(-1)
+        self.txid = str(txid)
         self.contract_name = None
         self.fn_name = name
 
@@ -163,26 +188,20 @@ class TransactionReceipt:
         self._revert_msg, self._revert_pc, revert_type = revert_data or (None, None, None)
         if self._revert_msg is None and revert_type not in ("revert", "invalid_opcode"):
             self._revert_msg = revert_type
+        if self._revert_pc is not None:
+            self._dev_revert_msg = build._get_dev_revert(self._revert_pc) or None
 
-        self._await_transaction(required_confs)
-
-        # if coverage evaluation is active, evaluate the trace
-        if (
-            CONFIG.argv["coverage"]
-            and not coverage._check_cached(self.coverage_hash)
-            and self.trace
-        ):
-            self._expand_trace()
+        self._await_transaction(required_confs, is_blocking)
 
     def __repr__(self) -> str:
-        c = {-1: "bright yellow", 0: "bright red", 1: None}
-        return f"<Transaction '{color(c[self.status])}{self.txid}{color}'>"
+        color_str = {-2: "dark white", -1: "bright yellow", 0: "bright red", 1: ""}[self.status]
+        return f"<Transaction '{color(color_str)}{self.txid}{color}'>"
 
     def __hash__(self) -> int:
         return hash(self.txid)
 
     @trace_property
-    def events(self) -> Optional[List]:
+    def events(self) -> Optional[EventDict]:
         if not self.status:
             self._get_trace()
         return self._events
@@ -229,6 +248,15 @@ class TransactionReceipt:
             self._get_trace()
         return self._revert_msg
 
+    @trace_property
+    def dev_revert_msg(self) -> Optional[str]:
+        if self.status:
+            return None
+        if self._dev_revert_msg is None:
+            self._get_trace()
+
+        return self._dev_revert_msg or None
+
     @trace_property
     def subcalls(self) -> Optional[List]:
         if self._subcalls is None:
@@ -243,7 +271,7 @@ class TransactionReceipt:
 
     @property
     def timestamp(self) -> Optional[int]:
-        if self.status == -1:
+        if self.status < 0:
             return None
         return web3.eth.getBlock(self.block_number)["timestamp"]
 
@@ -253,7 +281,59 @@ class TransactionReceipt:
             return 0
         return web3.eth.blockNumber - self.block_number + 1
 
+    def replace(
+        self,
+        increment: Optional[float] = None,
+        gas_price: Optional[Wei] = None,
+        silent: Optional[bool] = None,
+    ) -> "TransactionReceipt":
+        """
+        Rebroadcast this transaction with a higher gas price.
+
+        Exactly one of `increment` and `gas_price` must be given.
+
+        Arguments
+        ---------
+        increment : float, optional
+            Multiplier applied to the gas price of this transaction in order
+            to determine the new gas price
+        gas_price : Wei, optional
+            Absolute gas price to use in the replacement transaction
+        silent : bool, optional
+            Toggle console verbosity (default is same setting as this transaction)
+
+        Returns
+        -------
+        TransactionReceipt
+            New transaction object
+        """
+        if increment is None and gas_price is None:
+            raise ValueError("Must give one of `increment` or `gas_price`")
+        if gas_price is not None and increment is not None:
+            raise ValueError("Cannot set `increment` and `gas_price` together")
+        if self.status > -1:
+            raise ValueError("Transaction has already confirmed")
+
+        if increment is not None:
+            gas_price = Wei(self.gas_price * 1.1)
+
+        if silent is None:
+            silent = self._silent
+
+        return self.sender.transfer(  # type: ignore
+            self.receiver,
+            self.value,
+            gas_limit=self.gas_limit,
+            gas_price=Wei(gas_price),
+            data=self.input,
+            nonce=self.nonce,
+            required_confs=0,
+            silent=silent,
+        )
+
     def wait(self, required_confs: int) -> None:
+        if required_confs < 1:
+            return
         if self.confirmations > required_confs:
             print(f"This transaction already has {self.confirmations} confirmations.")
             return
@@ -263,14 +343,24 @@ class TransactionReceipt:
                 tx: Dict = web3.eth.getTransaction(self.txid)
                 break
             except TransactionNotFound:
-                time.sleep(0.5)
+                if self.nonce is not None:
+                    sender_nonce = web3.eth.getTransactionCount(str(self.sender))
+                    if sender_nonce > self.nonce:
+                        self.status = Status(-2)
+                        self._confirmed.set()
+                        return
+                time.sleep(1)
 
-        self._await_confirmation(tx, required_confs)
+        self._await_confirmation(tx["blockNumber"], required_confs)
 
     def _raise_if_reverted(self, exc: Any) -> None:
         if self.status or CONFIG.mode == "console":
             return
-        if self._revert_msg is None:
+        if not web3.supports_traces:
+            # if traces are not available, do not attempt to determine the revert reason
+            raise exc or ValueError("Execution reverted")
+
+        if self._dev_revert_msg is None:
             # no revert message and unable to check dev string - have to get trace
             self._expand_trace()
         if self.contract_address:
@@ -279,20 +369,28 @@ class TransactionReceipt:
             source = self._traceback_string()
         else:
             source = self._error_string(1)
-        raise exc._with_attr(source=source, revert_msg=self._revert_msg)
+        raise exc._with_attr(
+            source=source, revert_msg=self._revert_msg, dev_revert_msg=self._dev_revert_msg
+        )
 
-    def _await_transaction(self, required_confs: int = 1) -> None:
+    def _await_transaction(self, required_confs: int, is_blocking: bool) -> None:
         # await tx showing in mempool
         while True:
             try:
-                tx: Dict = web3.eth.getTransaction(self.txid)
+                tx: Dict = web3.eth.getTransaction(HexBytes(self.txid))
                 break
-            except TransactionNotFound:
+            except (TransactionNotFound, ValueError):
                 if self.sender is None:
                     # if sender was not explicitly set, this transaction was
                     # not broadcasted locally and so likely doesn't exist
                     raise
-                time.sleep(0.5)
+                if self.nonce is not None:
+                    sender_nonce = web3.eth.getTransactionCount(str(self.sender))
+                    if sender_nonce > self.nonce:
+                        self.status = Status(-2)
+                        return
+                time.sleep(1)
+
         self._set_from_tx(tx)
 
         if not self._silent:
@@ -302,27 +400,42 @@ class TransactionReceipt:
                 f"   Nonce: {color('bright blue')}{self.nonce}{color}"
             )
 
-        # await confirmation of tx in a separate thread which is blocking if required_confs > 0
+        # await confirmation of tx in a separate thread which is blocking if
+        # required_confs > 0 or tx has already confirmed (`blockNumber` != None)
         confirm_thread = threading.Thread(
-            target=self._await_confirmation, args=(tx, required_confs), daemon=True
+            target=self._await_confirmation, args=(tx["blockNumber"], required_confs), daemon=True
         )
         confirm_thread.start()
-        if required_confs > 0:
+        if is_blocking and (required_confs > 0 or tx["blockNumber"]):
             confirm_thread.join()
 
-    def _await_confirmation(self, tx: Dict, required_confs: int = 1) -> None:
-        if not tx["blockNumber"] and not self._silent and required_confs > 0:
+    def _await_confirmation(self, block_number: int = None, required_confs: int = 1) -> None:
+        block_number = block_number or self.block_number
+        if not block_number and not self._silent and required_confs > 0:
             if required_confs == 1:
-                print("Waiting for confirmation...")
+                sys.stdout.write("\rWaiting for confirmation... ")
             else:
                 sys.stdout.write(
-                    f"\rRequired confirmations: {color('bright yellow')}0/"
-                    f"{required_confs}{color}"
+                    f"\rRequired confirmations: {color('bright yellow')}0/{required_confs}{color}"
                 )
-                sys.stdout.flush()
+            sys.stdout.flush()
 
         # await first confirmation
-        receipt = web3.eth.waitForTransactionReceipt(self.txid, timeout=None, poll_latency=0.5)
+        while True:
+            # if sender nonce is greater than tx nonce, the tx should be confirmed
+            sender_nonce = web3.eth.getTransactionCount(str(self.sender))
+            expect_confirmed = bool(sender_nonce > self.nonce)  # type: ignore
+            try:
+                receipt = web3.eth.waitForTransactionReceipt(
+                    HexBytes(self.txid), timeout=15, poll_latency=1
+                )
+                break
+            except TimeExhausted:
+                if expect_confirmed:
+                    # if we expected confirmation based on the nonce, tx likely dropped
+                    self.status = Status(-2)
+                    self._confirmed.set()
+                    return
 
         self.block_number = receipt["blockNumber"]
         # wait for more confirmations if required and handle uncle blocks
@@ -338,7 +451,7 @@ class TransactionReceipt:
                 # check if tx is still in mempool, this will raise otherwise
                 tx = web3.eth.getTransaction(self.txid)
                 self.block_number = None
-                return self._await_confirmation(tx, required_confs)
+                return self._await_confirmation(tx["blockNumber"], required_confs)
             if required_confs - self.confirmations != remaining_confs:
                 remaining_confs = required_confs - self.confirmations
                 if not self._silent:
@@ -353,10 +466,24 @@ class TransactionReceipt:
                 time.sleep(1)
 
         self._set_from_receipt(receipt)
-        self._confirmed.set()
+        # if coverage evaluation is active, evaluate the trace
+        if (
+            CONFIG.argv["coverage"]
+            and not coverage._check_cached(self.coverage_hash)
+            and self.trace
+        ):
+            self._expand_trace()
         if not self._silent and required_confs > 0:
             print(self._confirm_output())
 
+        # set the confirmation event and mark other tx's with the same nonce as dropped
+        self._confirmed.set()
+        for dropped_tx in state.TxHistory().filter(
+            sender=self.sender, nonce=self.nonce, key=lambda k: k != self
+        ):
+            dropped_tx.status = Status(-2)
+            dropped_tx._confirmed.set()
+
     def _set_from_tx(self, tx: Dict) -> None:
         if not self.sender:
             self.sender = EthAddress(tx["from"])
@@ -379,7 +506,7 @@ class TransactionReceipt:
         self.txindex = receipt["transactionIndex"]
         self.gas_used = receipt["gasUsed"]
         self.logs = receipt["logs"]
-        self.status = receipt["status"]
+        self.status = Status(receipt["status"])
 
         self.contract_address = receipt["contractAddress"]
         if self.contract_address and not self.contract_name:
@@ -387,7 +514,7 @@ class TransactionReceipt:
 
         base = (
             f"{self.nonce}{self.block_number}{self.sender}{self.receiver}"
-            f"{self.value}{self.input}{self.status}{self.gas_used}{self.txindex}"
+            f"{self.value}{self.input}{int(self.status)}{self.gas_used}{self.txindex}"
         )
         self.coverage_hash = sha1(base.encode()).hexdigest()
 
@@ -399,9 +526,10 @@ class TransactionReceipt:
     def _confirm_output(self) -> str:
         status = ""
         if not self.status:
-            status = f"({color('bright red')}{self.revert_msg or 'reverted'}{color}) "
+            revert_msg = self.revert_msg if web3.supports_traces else None
+            status = f"({color('bright red')}{revert_msg or 'reverted'}{color}) "
         result = (
-            f"  {self._full_name()} confirmed {status}- "
+            f"\r  {self._full_name()} confirmed {status}- "
             f"Block: {color('bright blue')}{self.block_number}{color}   "
             f"Gas used: {color('bright blue')}{self.gas_used}{color} "
             f"({color('bright blue')}{self.gas_used / self.gas_limit:.2%}{color})"
@@ -427,15 +555,17 @@ class TransactionReceipt:
             self._trace = []
             return
 
+        if not web3.supports_traces:
+            raise RPCRequestError("Node client does not support `debug_traceTransaction`")
         try:
             trace = web3.provider.make_request(  # type: ignore
                 "debug_traceTransaction", (self.txid, {"disableStorage": CONFIG.mode != "console"})
             )
         except (requests.exceptions.Timeout, requests.exceptions.ConnectionError) as e:
             msg = f"Encountered a {type(e).__name__} while requesting "
-            msg += "debug_traceTransaction. The local RPC client has likely crashed."
+            msg += "`debug_traceTransaction`. The local RPC client has likely crashed."
             if CONFIG.argv["coverage"]:
-                msg += " If the error persists, add the skip_coverage fixture to this test."
+                msg += " If the error persists, add the `skip_coverage` marker to this test."
             raise RPCRequestError(msg) from None
 
         if "error" in trace:
@@ -471,7 +601,7 @@ class TransactionReceipt:
             fn = contract.get_method_object(self.input)
             self._return_value = fn.decode_output(data)
 
-    def _reverted_trace(self, trace: Sequence) -> None:
+    def _reverted_trace(self, trace: Sequence,) -> None:
         self._modified_state = False
         # get events from trace
         self._events = _decode_trace(trace, str(self.receiver or self.contract_address))
@@ -479,7 +609,9 @@ class TransactionReceipt:
             step = next((i for i in trace if i["op"] == "CODECOPY"), None)
             if step is not None and int(step["stack"][-3], 16) > 24577:
                 self._revert_msg = "exceeds EIP-170 size limit"
-        if self._revert_msg is not None:
+                self._dev_revert_msg = ""
+
+        if self._dev_revert_msg is not None:
             return
 
         # iterate over revert instructions in reverse to find revert message
@@ -488,27 +620,39 @@ class TransactionReceipt:
                 # get returned error string from stack
                 data = _get_memory(step, -1)[4:]
                 self._revert_msg = decode_abi(["string"], data)[0]
-                return
-            if self.contract_address:
+
+            elif self.contract_address:
                 self._revert_msg = "invalid opcode" if step["op"] == "INVALID" else ""
+                self._dev_revert_msg = ""
                 return
+
             # check for dev revert string using program counter
-            self._revert_msg = build._get_dev_revert(step["pc"])
+            dev_revert = build._get_dev_revert(step["pc"]) or None
+            if dev_revert is not None:
+                self._dev_revert_msg = dev_revert
+                if self._revert_msg is None:
+                    self._revert_msg = dev_revert
+            else:
+                # if none is found, expand the trace and get it from the pcMap
+                self._expand_trace()
+                try:
+                    pc_map = state._find_contract(step["address"])._build["pcMap"]
+                    # if this is the function selector revert, check for a jump
+                    if "first_revert" in pc_map[step["pc"]]:
+                        i = trace.index(step) - 4
+                        if trace[i]["pc"] != step["pc"] - 4:
+                            step = trace[i]
+                    self._dev_revert_msg = pc_map[step["pc"]]["dev"]
+                    if self._revert_msg is None:
+                        self._revert_msg = self._dev_revert_msg
+                    return
+                except (KeyError, AttributeError, TypeError):
+                    pass
+
             if self._revert_msg is not None:
+                if self._dev_revert_msg is None:
+                    self._dev_revert_msg = ""
                 return
-            # if none is found, expand the trace and get it from the pcMap
-            self._expand_trace()
-            try:
-                pc_map = state._find_contract(step["address"])._build["pcMap"]
-                # if this is the function selector revert, check for a jump
-                if "first_revert" in pc_map[step["pc"]]:
-                    i = trace.index(step) - 4
-                    if trace[i]["pc"] != step["pc"] - 4:
-                        step = trace[i]
-                self._revert_msg = pc_map[step["pc"]]["dev"]
-                return
-            except (KeyError, AttributeError, TypeError):
-                pass
 
         step = next(i for i in trace[::-1] if i["op"] in ("REVERT", "INVALID"))
         self._revert_msg = "invalid opcode" if step["op"] == "INVALID" else ""
@@ -645,9 +789,12 @@ class TransactionReceipt:
                     subcall["selfdestruct"] = True
                 else:
                     if opcode == "REVERT":
-                        data = _get_memory(trace[i], -1)[4:]
-                        if data:
-                            subcall["revert_msg"] = decode_abi(["string"], data)[0]
+                        data = _get_memory(trace[i], -1)
+                        if len(data) > 4:
+                            try:
+                                subcall["revert_msg"] = decode_abi(["string"], data[4:])[0]
+                            except Exception:
+                                subcall["revert_msg"] = data.hex()
                     if "revert_msg" not in subcall and "dev" in pc:
                         subcall["revert_msg"] = pc["dev"]
 
@@ -1087,7 +1234,10 @@ def _step_external(
 def _get_memory(step: Dict, idx: int) -> HexBytes:
     offset = int(step["stack"][idx], 16)
     length = int(step["stack"][idx - 1], 16)
-    return HexBytes("".join(step["memory"]))[offset : offset + length]
+    data = HexBytes("".join(step["memory"]))[offset : offset + length]
+    # append zero-bytes if allocated memory ends before `length` bytes
+    data = HexBytes(data + b"\x00" * (length - len(data)))
+    return data
 
 
 def _get_last_map(address: EthAddress, sig: str) -> Dict:
